function varargout = ma_gui(varargin)
% MA_GUI Application M-file for ma_gui.fig
%    FIG = MA_GUI launch ma_gui GUI.
%    MA_GUI('callback_name', ...) invoke the named callback.

% Musik Analysis Toolbox for Matlab Version 0.2
%              Copyright (c) 2002 Elias Pampalk
%                   Austrian Research Institute 
%                   for Artificial Intelligence
%                   http://www.oefai.at/~elias/

% this file contains:
% - feature extraction
%   - loudness sensation
%   - fluctuation strength
%   - median (typical rhythm pattern)
% - organization
%   - pca
%   - som
% - visualization
%   - sdh
%   - html output generator (type: imagemap)

if nargin == 0  % LAUNCH GUI
    
    fig = openfig(mfilename,'reuse');
    
    % Generate a structure of handles to pass to callbacks, and store it. 
    handles = guihandles(fig);
    guidata(fig, handles);
    
    if nargout > 0
        varargout{1} = fig;
    end
    
elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK
    
    try
        if (nargout)
            [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
        else
            feval(varargin{:}); % FEVAL switchyard
        end
    catch
        disp(lasterr);
    end
    
end

% --------------------------------------------------------------------
function varargout = checkbox1_Callback(h, eventdata, handles, varargin)
if get(handles.chkbox_auto,'value'),
    set(handles.edit_X,'enable','off');
    set(handles.edit_X,'BackgroundColor',[0.8 0.8 0.8]);
    set(handles.edit_X,'ForegroundColor',[0.5 0.5 0.5]);
    
    set(handles.edit_Y,'enable','off');
    set(handles.edit_Y,'BackgroundColor',[0.8 0.8 0.8]);
    set(handles.edit_Y,'ForegroundColor',[0.5 0.5 0.5]);
else
    set(handles.edit_X,'enable','on');
    set(handles.edit_X,'BackgroundColor',[1 1 1]);
    set(handles.edit_X,'ForegroundColor',[0 0 1]);
    
    set(handles.edit_Y,'enable','on');
    set(handles.edit_Y,'BackgroundColor',[1 1 1]);
    set(handles.edit_Y,'ForegroundColor',[0 0 1]);
end

% --------------------------------------------------------------------
function varargout = pushbutton1_Callback(h, eventdata, handles, varargin)
% start action
c = [0.753 0.753 0.753];
set(handles.frame1,'backgroundcolor',c);
set(handles.txt_wav,'backgroundcolor',c);
set(handles.txt_mp3,'backgroundcolor',c);
set(handles.txt_html,'backgroundcolor',c);
set(handles.txt_som,'backgroundcolor',c);
set(handles.chkbox_auto,'backgroundcolor',c);
set(handles.txt_X,'backgroundcolor',c);
set(handles.txt_Y,'backgroundcolor',c);

set(handles.edit_html,'enable','off')
set(handles.edit_mp3,'enable','off')
set(handles.edit_wav,'enable','off')
set(handles.chkbox_auto,'enable','off')
set(handles.edit_X,'enable','off')
set(handles.edit_Y,'enable','off')

set(handles.btn_start,'enable','off')

c = [0.8,0.804,0.857];
set(handles.frame2,'backgroundcolor',c);
set(handles.txt1,'backgroundcolor',c);
set(handles.txt2,'backgroundcolor',c);
set(handles.txt3,'backgroundcolor',c);
set(handles.txt4,'backgroundcolor',c);
set(handles.txt5,'backgroundcolor',c);
set(handles.txt6,'backgroundcolor',c);
set(handles.txt7,'backgroundcolor',c);
drawnow

buffertext('> Check Input Arguments',handles)

[a,b]=system([get(handles.edit_wav,'string'),' --version']);
if ~a,
    buffertext('  - Lame exists: yes.',handles);
else
    buffertext('  - Lame exists: no.   FATAL ERROR!',handles);
    restore(handles);
    return
end

str_mp3dir=get(handles.edit_mp3,'string');
if ~strcmp(str_mp3dir(end),'\') & ~strcmp(str_mp3dir(end),'/'),
    str_mp3dir(end+1)='/';
end
d=dir([str_mp3dir,'*.mp3']);
labels = char(d.name);
n=size(labels,1);
if n>0,
    buffertext(['  - MP3s exist: yes. (Found ',num2str(n),' MP3s)'],handles);
else
    buffertext('  - MP3s exist: no.   FATAL ERROR!',handles);
    restore(handles);
    return
end

html_str=get(handles.edit_html,'string');
if ~strcmp(html_str(end),'\') & ~strcmp(html_str(end),'/'),
    html_str(end+1)='/';
end

dummy = mkdir(html_str,'images');
if ~dummy,
    buffertext('  - HTML directory access rights: no.   FATAL FAILURE!',handles);
    restore(handles);
    return
else
    buffertext('  - HTML directory access rights: yes.',handles);
end

if get(handles.chkbox_auto,'value'),
    y = max(2,ceil(sqrt(n)*0.8/1.4));
    x = max(2,ceil(sqrt(n)*0.6/1.4));
    buffertext(['  - SOM parameters: auto. (X = ',num2str(y),', Y = ',num2str(x),')'],handles);
else
    y = str2num(get(handles.edit_X,'string'));
    x = str2num(get(handles.edit_Y,'string'));
    if any([isnumeric(x),isnumeric(y),x>1,y>1,round(x) == x,round(y) == y]==0),
        buffertext('  - SOM parameters: inappropiate.   FATAL ERROR!',handles);
        restore(handles);
        return
    end
    set(handles.txt5,'string','  - SOM parameters: ok.');
end

global SIGNAL_QUIT
SIGNAL_QUIT=0;
buffertext('> Start Feature Extraction',handles);
drawnow

sData = ma_gui_feature_extraction(handles,labels,x,y,html_str,str_mp3dir);

buffertext('-- DONE.',handles);


% --------------------------------------------------------------------
function pushbutton3_Callback(h, eventdata, handles, varargin)
global SIGNAL_QUIT
SIGNAL_QUIT=1;
close(handles.figure1);


% --------------------------------------------------------------------
function varargout = restore(handles)
c = [0.8,0.804,0.857];
set(handles.frame1,'backgroundcolor',c);
set(handles.txt_wav,'backgroundcolor',c);
set(handles.txt_mp3,'backgroundcolor',c);
set(handles.txt_html,'backgroundcolor',c);
set(handles.txt_som,'backgroundcolor',c);
set(handles.chkbox_auto,'backgroundcolor',c);
set(handles.txt_X,'backgroundcolor',c);
set(handles.txt_Y,'backgroundcolor',c);

set(handles.edit_html,'enable','on')
set(handles.edit_mp3,'enable','on')
set(handles.edit_wav,'enable','on')
set(handles.chkbox_auto,'enable','on')
if ~get(handles.chkbox_auto,'value'),
    set(handles.edit_X,'enable','on')
    set(handles.edit_Y,'enable','on')
end

set(handles.btn_start,'enable','on')

c = [0.753 0.753 0.753];
set(handles.frame2,'backgroundcolor',c);
set(handles.txt1,'backgroundcolor',c);
set(handles.txt2,'backgroundcolor',c);
set(handles.txt3,'backgroundcolor',c);
set(handles.txt4,'backgroundcolor',c);
set(handles.txt5,'backgroundcolor',c);
set(handles.txt6,'backgroundcolor',c);
set(handles.txt7,'backgroundcolor',c);

% --------------------------------------------------------------------
function buffertext(str,h)
set(h.txt1,'string',get(h.txt2,'string'));
set(h.txt2,'string',get(h.txt3,'string'));
set(h.txt3,'string',get(h.txt4,'string'));
set(h.txt4,'string',get(h.txt5,'string'));
set(h.txt5,'string',get(h.txt6,'string'));
set(h.txt6,'string',get(h.txt7,'string'));
set(h.txt7,'string',str);

% --------------------------------------------------------------------
function sData = ma_gui_feature_extraction(handles,fis,x,y,html_str,str_mp3dir)
global SIGNAL_QUIT

sData = [];

avg_seq_per_song = 7; % opt mem
si = 1;
data = zeros(size(fis,1)*avg_seq_per_song,600); % opt mem
labels = [];

f = 22050/256*(0:512/2)/512;
c.w = repmat(1./(f(2:41)/4+4./f(2:41)),15,1);
h = [.05 .1 0.25 0.5 1 0.5 .25 0.1 0.05];

c.blu = filter2(h,eye(15));
c.blu = c.blu./repmat(sum(c.blu,2),1,15);
c.blu2 = filter2(h,eye(40)); 
c.blu2 = c.blu2./repmat(sum(c.blu2,2),1,40);

c.phons(1)=3;
c.phons(2:6)=20:20:100;

c.bark = [100 200 300 400 510 630 770 920 1080 1270 1480 1720 2000 2320 2700 3150 3700 4400 5300 6400 7700 9500];
c.f = [31.62, 50, 70.7, 100, 141.4, 200, 316.2, 500, 707.1, 1000, 1414, 1682, 2000, 2515, 3162, 3976, 5000, 7071, 10000, 11890, 14140, 15500];
c.equal_loudness = [
    55   40   32   24   19   14   10    6    4    3    2    2    0   -2   -5   -4    0    5   10   14   25   35; 
    66   52   43   37   32   27   23   21   20   20   20   20   19   16   13   13   18   22   25   30   40   50;
    76   64   57   51   47   43   41   41   40   40   40   39.5 38   35   33   33   35   41   46   50   60   70;
    89   79   74   70   66   63   61   60   60   60   60   59   56   53   52   53   56   61   65   70   80   90;
    103   96   92   88   85   83   81   80   80   80   80   79   76   72   70   70   75   79   83   87   95  105;
    118  110  107  105  103  102  101  100  100  100  100   99   97   94   90   90   95  100  103  105  108  115];

i=1; j=1;
for bsi = c.bark,
    while j<length(c.f) & bsi>c.f(j),
        j=j+1;
    end
    j=j-1;
    if ~isempty(find(c.f==bsi)),
        c.bs_val(:,i) = c.equal_loudness(:,find(c.f==bsi));
    else
        w1 = 1 / abs(c.f(j) - bsi);
        w2 = 1 / abs(c.f(j + 1) - bsi);
        c.bs_val(:,i) = (c.equal_loudness(:,j).*w1 + c.equal_loudness(:,j+1).*w2) / (w1 + w2);
    end
    i=i+1;
end

for i = 1:15,
    c.spread(i,:) = 10.^((15.81+7.5*((i-(1:15))+0.474)-17.5*(1+((i-(1:15))+0.474).^2).^0.5)/10);
end

tic;
seq_idx = zeros(size(data,1),1);
for sj = 1:size(fis,1),
    sk = 3;
    
    t1 = toc;
    
    if sj==1,
        buffertext(['   0% Done. Loading: ',fis(sj,:)],handles);
    else
        t_expect = round(t1/(sj-1)*(size(fis,1)-sj+1)/60);
        buffertext(['   ',num2str(round((sj-1)/size(fis,1)*100)),'% Done. Loading: ',strcat(fis(sj,:)),' (Maybe ', num2str(t_expect),' more minutes.)'],handles);
    end
    
    if SIGNAL_QUIT,
        disp('Killed.')
        return
    end
    
    system([get(handles.edit_wav,'string'),' --quiet --decode -m m --resample 11.025 ',strcat(get(handles.edit_mp3,'string'),fis(sj,:)),' ./tmp.wav'])
    siz = wavread('./tmp.wav','size');
    
    
    s = [1 2^16];
    while s(2)+(sk-1)*2^16 < siz(1),
        pause(0.01); % UI control
        if SIGNAL_QUIT,
            disp('Killed.')
            return
        end
        d = [];
        
        [wav, fs, nbits] = wavread('./tmp.wav',s+(sk-1)*2^16);
        
        seq_number = num2str(sk);
        if sk<10,
            seq_number = ['0',seq_number];
        end
        wav = 0.35*wav*2^15;
        
        w = hann(256); % dsp
        d1 = zeros(129,511);
        idx = 1:256;
        for i=1:511,
            d1(:,i) = periodogram(wav(idx),w);
            idx = idx + 128;
        end
        
        freq_axis = 11025/256*(0:128);
        d2 = zeros(15, 511);
        d2(1,:) = sum(d1(find(freq_axis < c.bark(1)),:),1);
        for i = 2:15,
            d2(i,:) = sum(d1(find(freq_axis > c.bark(i-1) & freq_axis < c.bark(i)),:));
        end
        
        d2 = c.spread*d2;
        d2(d2<1) = 1;
        d2 = 10*log10(d2);
        
        for b = 1:15,
            for t = 1:511,
                level = 1;
                while d2(b,t) > c.bs_val(level,b),
                    level = level + 1;
                    if level>6, 
                        error('level > 6'); 
                    end
                end
                if level == 1,
                    d2(b,t) = c.phons(1);
                else
                    d1 = (d2(b,t)-c.bs_val(level-1,b))/(c.bs_val(level,b)-c.bs_val(level-1,b));
                    d2(b,t) = c.phons(level-1) + d1*(c.phons(level)-c.phons(level-1));
                end    
            end
        end
        
        idx = d2>=40;
        d2(idx) = 2.^((d2(idx)-40)/10);
        d2(~idx) = (d2(~idx)/40).^2.642;
        
        d3 = zeros(15,40);
        for b = 1:15,
            X = fft([d2(b,:),0],512);
            d3(b,:) = abs(X(2:41))/256;
        end
        
        d3 = d3.*c.w;
        
        for i=2:40,
            d3(:,i-1) = abs(d3(:,i)-d3(:,i-1));
        end
        d3 = c.blu*d3*c.blu2;
        
        d = d3(:);
        
        data(si,:) = d'; 
        seq_idx(si) = sj;
        si = si + 1;
        sk = sk + 3;
    end
    delete('./tmp.wav');
end

data = data(1:si-1,:); % opt mem
seq_idx = seq_idx(1:si-1,:);

buffertext(['  100% Loaded. Total Time: ',num2str(round(toc/60)),' minutes'],handles);

buffertext('  Calculating Median',handles);
song = zeros(max(seq_idx),600);
for i=1:max(seq_idx)
    idx = find(seq_idx==i);
    song(i,:)=median(data(idx,:),1);
    song_labels{i} = fis(i,1:max(findstr(fis(i,:),'.'))-1);
end

buffertext('> Start Organization',handles);
buffertext('  Calculating PCA',handles);

mc = mean(song);
song_mc = song-repmat(mc,size(song,1),1);
[eVts, eVls] = eig(cov(song_mc));

eigvecs = min(size(song,1),80);
D = eVts(:,end-eigvecs+1:end)'*song_mc';

buffertext('  Calculating SOM',handles);

radius=[3*max(x,y),linspace(max(x,y),ceil(max(x,y)/4),50),linspace(ceil(max(x,y)/4),0.1,100)];

m = x * y;       % number of map units
[n d] = size(D); % number of dataitems and their dimensions

M=rand(m,d)+repmat(mean(D),m,1); % init with small values
Cxy = [reshape(repmat(1:x,y,1),1,m); repmat(y:-1:1,1,x)]'; % rectangular order
Ud = zeros(m,m); 
for i = 1:(m-1), Ud(i,i+1:m)=sum((Cxy(i+1:m,:)-repmat(Cxy(i,:),m-i,1))'.^2); end
Ud = Ud + Ud'; % symmetric

D2=2*D'; % optimize training loop
for t = 1:length(radius) % training
    [Q W] = min(repmat(sum(M.^2,2),1,n)-M*D2);   % quantisation error, winner (unit with smallest to data item)
    P = sparse(W,1:n,ones(n,1),m,n);             % partition P(i,j)=1 if unit i is activated by dataitem j else P(i,j)=0
    N = exp(-Ud/(2*radius(t)));                  % gaussian neighborhood function
    S = N*(P*D);                                 % weighted sum of mapped winners
    A = N*sum(P,2);                              % activation normalization factor
    nonzero = find(A > 0);                       % update only units with activation > 0
    M(nonzero,:) = S(nonzero,:) ./ repmat(A(nonzero),1,d); 
end

[Q W] = min(repmat(sum(M.^2,2),1,n)-M*D2);
for i=1:m,
    idx = find(W==i);
    if ~isempty(idx),
        map_labels{i} = song_labels(idx);
    else
        map_labels{i} = [];
    end
end

buffertext('  Calculating SDH',handles);

rows = x;
cols = y;
[vecs dims] = size(D);

Dist = repmat(sum(M.^2,2),1,vecs)-M*2*D'; % munits x vecs

spread = 3;

m=calc_spread(spread,Dist,m,rows,cols,vecs);

f = figure;
set(f,'numbertitle','off');
set(f,'name','Islands of Music');
s=subplot(1,1,1);
%imagesc(m)
pcolor(m); shading interp
set(s,'ydir','normal')
colormap(iom_colormap);
hold on; 

x = cols;
y = rows;
c = 2^2; % interp

fr = 1; % frame
o=fr*c+1;

e = zeros(x*y,4);
k=1;
for j=1:x,
    for i=1:y,
        
        str = map_labels{k};
        
        if ~isempty(str)
            h=text(o+(j-1)*c, o+(i-1)*c, ...
                str,'fontsize',8,'horizontalalignment','center','color','r'); 
            set(h,'units','normalized');
            e(k,:) = get(h,'extent');
            set(h,'units','data');
        end
        %plot(o+(j-1)*c, o+(i-1)*c,'ro')
        
        if 0
            lt='r';
            lux = e(k,1);
            luy = e(k,2)+(j-1)*e(i,4)/k;
            
            rox = lux + e(k,3);
            roy = luy + e(k,4)/k;
            
            lux=lux*37;
            luy=luy*29;
            rox=rox*37;
            roy=roy*29;
            
            plot([lux rox], [luy luy],lt); hold on
            plot([lux lux], [luy roy],lt)
            plot([rox lux], [roy roy],lt)
            plot([rox rox], [luy roy],lt)
        end
        k=k+1;
    end
end

set(s,'ydir','reverse');
set(s,'xtick',[-realmax realmax]);
set(s,'ytick',[-realmax realmax]);

set(gcf,'renderer','painter')
set(gcf,'menubar','none')

buffertext('  Creating HTML Output',handles);

mx=size(m,2);
my=size(m,1);

screensize = get(0,'screensize')/2;
cx = screensize(3);
cy = screensize(4);
w_hor=300;
w_vert=300;
if my<mx,
    w_hor = w_hor*my/mx;
else
    w_vert = w_vert*mx/my;
end

max_x = w_vert*2;
max_y = w_hor*2;

set(gca,'position',[0 0 1 1])
set(gcf,'position',[cx-w_vert,cy-w_hor,max_x,max_y]);

set(gcf,'paperpositionmode','auto')

fid=fopen([html_str,'index.html'],'w');

fprintf(fid,'<html>\n');
fprintf(fid,'<head>\n');
fprintf(fid,'<title>Islands of Music (Version 0.1)\n');
fprintf(fid,'</title>\n');
fprintf(fid,'</head>\n');

fprintf(fid,'<body>\n');
fprintf(fid,'<center>\n');
fprintf(fid,'<MAP NAME="bigmap">\n');

lt='r';
e2=e;
e(:,3:4)=e(:,3:4)*3/5;
e(:,1:2)=e(:,1:2)+e(:,3:4)/3;

%%
idx = reshape(1:prod(x*y),y,x);
%idx = flipud(reshape(1:prod(x*y),y,x));

for i=1:size(e,1);
    k=length(map_labels{i});
    for j=1:k,
        
        lux = e(idx(i),1);
        luy = e(idx(i),2)+(j-1)*e(idx(i),4)/k;
        
        rox = lux + e(idx(i),3);
        roy = luy + e(idx(i),4)/k;
        
        if 0
            plot([lux rox], [luy luy],lt); hold on
            plot([lux lux], [luy roy],lt)
            plot([rox lux], [roy roy],lt)
            plot([rox rox], [luy roy],lt)
        end
        
        fprintf(fid,'<AREA HREF="%s%s.mp3" ALT="%s" TITLE="%s" SHAPE=RECT COORDS="%d,%d,%d,%d">\n',str_mp3dir,char(map_labels{i}(j)),char(map_labels{i}(j)),char(map_labels{i}(j)),round(lux*max_x),round(luy*max_y),round(rox*max_x),round(roy*max_y));
    end
end
e=e2;

eval(['print -dpng -r0 ',html_str,'images/iom.png'])


fprintf(fid,'</MAP>\n');
fprintf(fid,'<IMG SRC="./images/iom.png" BORDER=0 USEMAP="#bigmap"><BR>\n');
fprintf(fid,'<BR><BR>Created by <a href="http://www.oefai.at/~elias/music"><b>Islands of Music (Version 0.1)</b></a>.<BR>');
fprintf(fid,'(c) 2002 Elias Pampalk (elias@oefai.at)<BR>');


fprintf(fid,'</body>\n');
fprintf(fid,'</html>\n');

fclose(fid);


% --------------------------------------------------------------------
function m=calc_spread(spread,Dist,m,rows,cols,vecs)

m = zeros(rows*cols,1);

c = (spread:-1:1)';
for i=1:vecs,
    [d idx] = sort(Dist(:,i));
    idx = idx(1:spread); 
    m(idx) = m(idx) + c;
end
m = reshape(m,rows,cols);

FRACT = 0.1;

m2 = zeros(size(m,1)+2,size(m,2)+2);
c = min(m(:)); % don't set values smaller than this

% corners: FRACTion of old corner value
m2(1,1)     = max(c,FRACT*m(1,1));     % top left
m2(end,1)   = max(c,FRACT*m(end,1));   % bottom left
m2(1,end)   = max(c,FRACT*m(1,end));   % top right
m2(end,end) = max(c,FRACT*m(end,end)); % bottom right

% center: not changed
m2(2:end-1,2:end-1) = m;

% edges: 0.1 of old edges
m2(2:end-1,1)   = max(c,FRACT*m(:,1));   % left
m2(2:end-1,end) = max(c,FRACT*m(:,end)); % right
m2(1,2:end-1)   = max(c,FRACT*m(1,:));   % top
m2(end,2:end-1) = max(c,FRACT*m(end,:)); % bottom

m = interp2(m2,2,'spline');

% --------------------------------------------------------------------
function rgb = iom_colormap
sea_level = round(64/3);

h=[...
        linspace(1,1,sea_level)*2/3,...
        60/360,60/360,...            
        ones(1,42-sea_level)*1/3,...        
        ones(1,12)*1/3,... 
        ones(1,8)]';       
s=[...
        linspace(1,1,sea_level),...     
        0.7,0.7,...           
        ones(1,42-sea_level),...     
        linspace(1,0.5,12),...     
        zeros(1,8)]';       
v=[...
        linspace(0.5,1,sea_level),...       % water    (sea level)
        1,1,...                             % beach    (02)
        linspace(0.4,0.6,42-sea_level),...  % forest   (64 - 20 - sea level))
        linspace(0.6,0.7,12),...            % gras     (12)    
        linspace(0.7,1,8)]';                % mountain (10)    

rgb = hsv2rgb([h,s,v]);
